/*******************************************************************************
 * Copyright 2007 SWTBot, http://swtbot.org/
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package net.sf.swtbot.eclipse.finder;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.sf.swtbot.SWTBot;
import net.sf.swtbot.eclipse.finder.resolver.EclipseDefaultChildrenResolver;
import net.sf.swtbot.eclipse.finder.resolver.EclipseDefaultParentResolver;
import net.sf.swtbot.eclipse.finder.widgets.SWTBotEclipseEditor;
import net.sf.swtbot.eclipse.finder.widgets.SWTBotView;
import net.sf.swtbot.finder.ControlFinder;
import net.sf.swtbot.finder.MenuFinder;
import net.sf.swtbot.widgets.WidgetNotFoundException;

import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IViewReference;

/**
 * This extends the {@link SWTBot} and adds specific capabilities for testsing
 * eclipse based items (Views, Editors).
 * 
 * @author Ketan Padegaonkar &lt;KetanPadegaonkar [at] gmail [dot] com&gt;
 * @version $Id: SWTEclipseBot.java 757 2008-06-21 04:04:07Z kpadegaonkar $
 */
public class SWTEclipseBot extends SWTBot {

	/**
	 * Constructs an eclipse bot.
	 */
	public SWTEclipseBot() {
		super(new ControlFinder(new EclipseDefaultChildrenResolver(), new EclipseDefaultParentResolver()), new MenuFinder());
	}

	/**
	 * Attempts to locate the editor matching the given name.  If no match 
	 * is found an exception will be thrown.  The name is the name as displayed
	 * on the editor's tab in eclipse.
	 * 
	 * @param fileName the name of the file.
	 * @return an editor for the specified fileName.
	 * @throws WidgetNotFoundException if the editor is not found.
	 */
	public SWTBotEclipseEditor editor(String fileName) throws WidgetNotFoundException {
		return new SWTBotEclipseEditor(this, finder, fileName);
	}

	/**
	 * Attempts to find the view matching the given label.  If no match is 
	 * found then an exception will be thrown. The name is the name as displayed
	 * on the editor's tab in eclipse.
	 * 
	 * @param name the name of the view.
	 * @return a view with the specified label.
	 * @throws WidgetNotFoundException if the view is not found.
	 */
	public SWTBotView view(String name) throws WidgetNotFoundException {
		return new SWTBotView(finder, name);
	}

	/**
	 * Returns the list of all the open editors found in the active workbench.
	 * 
	 * @return all the editors in the workbench.
	 * @throws WidgetNotFoundException if there are errors finding editors.
	 */
	public List editors() throws WidgetNotFoundException {
		List editors = SWTBotView.findEditors();

		List result = new ArrayList(editors.size());
		for (Iterator iterator = editors.iterator(); iterator.hasNext();) {
			IEditorReference editorReference = (IEditorReference) iterator.next();
			result.add(new SWTBotEclipseEditor(editorReference));
		}
		return result;
	}

	/**
	 * Returns the list of all the open views found in the active workbench.
	 * 
	 * @return all the views in the workbench.
	 * @throws WidgetNotFoundException if the views are not found.
	 */
	public List views() throws WidgetNotFoundException {
		List editors = SWTBotView.findViews();

		List result = new ArrayList(editors.size());
		for (Iterator iterator = editors.iterator(); iterator.hasNext();) {
			IViewReference viewReference = (IViewReference) iterator.next();
			result.add(new SWTBotView(viewReference));
		}
		return result;
	}

	/**
	 * Return the active editor.
	 * 
	 * @return the active editor, if any
	 * @throws WidgetNotFoundException if there is no active editor.
	 * @see SWTBotView#activeEditor()
	 * @since 1.1
	 */
	public SWTBotEclipseEditor activeEditor() throws WidgetNotFoundException {
		return SWTBotView.activeEditor();
	}
}
