/*******************************************************************************
 * Copyright 2007 SWTBot, http://swtbot.org/
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package net.sf.swtbot.eclipse.finder.widgets;

import java.util.ArrayList;
import java.util.List;

import net.sf.swtbot.SWTBot;
import net.sf.swtbot.eclipse.finder.SWTEclipseBot;
import net.sf.swtbot.eclipse.finder.exceptions.QuickFixNotFoundException;
import net.sf.swtbot.finder.ChildrenControlFinder;
import net.sf.swtbot.finder.Finder;
import net.sf.swtbot.finder.MenuFinder;
import net.sf.swtbot.finder.UIThreadRunnable.ListResult;
import net.sf.swtbot.finder.UIThreadRunnable.VoidResult;
import net.sf.swtbot.finder.UIThreadRunnable.WidgetResult;
import net.sf.swtbot.matcher.ClassMatcher;
import net.sf.swtbot.matcher.IMatcher;
import net.sf.swtbot.utils.Position;
import net.sf.swtbot.widgets.SWTBotMenu;
import net.sf.swtbot.widgets.SWTBotShell;
import net.sf.swtbot.widgets.SWTBotStyledText;
import net.sf.swtbot.widgets.SWTBotTable;
import net.sf.swtbot.widgets.TimeoutException;
import net.sf.swtbot.widgets.WidgetNotFoundException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.Bullet;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;

/** This represents an eclipse editor item.
 * @author Ketan Padegaonkar &lt;KetanPadegaonkar [at] gmail [dot] com&gt;
 * @version $Id: SWTBotEclipseEditor.java 809 2008-06-29 04:05:10Z kpadegaonkar $
 */
public class SWTBotEclipseEditor extends SWTBotView {

	/**  The bot to use.*/
	private final SWTBot		bot;
	/**  The styled text of the editor.*/
	private SWTBotStyledText	styledText;

	/**
	 * Constructs an instance of this object.
	 * @param bot the bot
	 * @param finder the finder used to find controls.
	 * @param text the text on the control.
	 * @throws WidgetNotFoundException if the widget is not found.
	 */
	public SWTBotEclipseEditor(SWTBot bot, Finder finder, String text) throws WidgetNotFoundException {
		super(finder, text);
		this.bot = bot;
	}

	/**
	 * This constructor calls {@link #findControls(IMatcher)} to find all controls matching {@link #matcher()}, in case
	 * a widget is not found, the constructor retries until a timeout occurs. The first such control is set into
	 * {@link #widget} the other controls, if any that match {@link #matcher()} are held in the {@link #similarWidgets}
	 * list.
	 * 
	 * @see SWTBot#waitUntil(net.sf.swtbot.wait.ICondition)
	 * @param finder the finder used to find controls.
	 * @param text the text on the control.
	 * @param index the index of the control in case multiple controls are found.
	 * @throws WidgetNotFoundException if the widget is not found.
	 * @since 1.0
	 */
	public SWTBotEclipseEditor(Finder finder, String text, int index) throws WidgetNotFoundException {
		super(finder, text, index);
		bot = new SWTEclipseBot();
	}

	/**
	 * Constructs an instnace of the given object.
	 * 
	 * @param editorReference the editor reference.
	 * @throws WidgetNotFoundException if the widget is <code>null</code> or widget has been disposed.
	 */
	public SWTBotEclipseEditor(IEditorReference editorReference) throws WidgetNotFoundException {
		super(editorReference);
		this.widget = findWidget(0);
		bot = new SWTEclipseBot();
	}

	/**
	 * @see net.sf.swtbot.eclipse.finder.widgets.SWTBotView#findWidget(int)
	 */
	protected Widget findWidget(int index) throws WidgetNotFoundException {
		Widget findWidget = findWidget(view, new Finder(new ChildrenControlFinder(super.findWidget(index)), new MenuFinder()));
		return getEditor((Composite) findWidget);
	}

	/**
	 * @param composite the composite containing the styled text.
	 * @return the style text editor in the editor.
	 * @throws WidgetNotFoundException if the styled text could not be found.
	 */
	protected StyledText getEditor(final Composite composite) throws WidgetNotFoundException {
		StyledText editor = findEditor(composite);
		styledText = new SWTBotStyledText(editor);
		return editor;
	}

	/**
	 * Finds the editor using the given composite.
	 * 
	 * @param composite The composite to use.
	 * @return The {@link StyledText} item.
	 */
	private StyledText findEditor(final Composite composite) {
		final ClassMatcher classMatcher = new ClassMatcher(StyledText.class);
		return (StyledText) syncExec(new WidgetResult() {
			public Widget run() {
				StyledText styledText = null;
				try {
					List findControls = finder.findControls(composite, classMatcher, true);
					styledText = (StyledText) findControls.get(0);
				} catch (Exception e) {
					log.error(e);
				}
				return styledText;
			}
		});
	}

	/**
	 * Applys a quick fix item at the given index.
	 * 
	 * @param quickFixIndex the index of the quickfix item to apply.
	 * @throws QuickFixNotFoundException if the quickfix could not be found.
	 * @throws WidgetNotFoundException if the quickfix could not be found.
	 */
	public void quickfix(int quickFixIndex) throws QuickFixNotFoundException, WidgetNotFoundException {
		SWTBotShell quickFixShell = activateQuickFixShell();
		SWTBotTable quickFixTable = getQuickFixTable(quickFixShell);
		applyQuickFix(quickFixTable, quickFixIndex);
	}

	/**
	 * Applys a quick fix item with the given name.
	 * 
	 * @param quickFixName the name of the quick fix to apply.
	 * @throws QuickFixNotFoundException if the quickfix could not be found.
	 */
	public void quickfix(String quickFixName) throws QuickFixNotFoundException {
		int retries = 10;
		SWTBotShell quickFixShell = activateQuickFixShell();
		SWTBotTable quickFixTable = getQuickFixTable(quickFixShell);
		applyQuickFix(quickFixTable, quickFixName, retries);
	}

	/**
	 * Finds all the quickfixes in the quickfix list.
	 * 
	 * @return the list of all available quickfixes.
	 * @throws QuickFixNotFoundException if the quickfix could not be found.
	 * @since 1.2
	 */
	public List getQuickFixes() throws QuickFixNotFoundException {
		SWTBotShell quickFixShell = activateQuickFixShell();
		SWTBotTable quickFixTable = getQuickFixTable(quickFixShell);
		int rowCount = quickFixTable.rowCount();
		List result = new ArrayList();
		for (int i = 0; i < rowCount; i++)
			result.add(quickFixTable.cell(i, 0));
		return result;
	}

	/**
	 * Gets the quick fix list item count.
	 * <p>
	 * The getQuickfixListItemCount should be used instead.
	 * </p>
	 * 
	 * @return the number of quickfix items in the quickfix proposals.
	 * @throws QuickFixNotFoundException if the quickfix could not be found.
	 * @since 1.0
	 * @deprecated Use {@link #getQuickfixListItemCount()} instead
	 */
	public int quickfixListItemCount() throws QuickFixNotFoundException {
		return getQuickfixListItemCount();
	}

	/**
	 * Gets the quick fix item count.
	 * 
	 * @return the number of quickfix items in the quickfix proposals.
	 * @throws QuickFixNotFoundException if the quickfix could not be found.
	 * @since 1.2
	 */
	public int getQuickfixListItemCount() throws QuickFixNotFoundException {
		SWTBotShell quickFixShell = activateQuickFixShell();
		SWTBotTable quickFixTable = getQuickFixTable(quickFixShell);
		return quickFixTable.rowCount();
	}

	/**
	 * Attempst to applys the quick fix.
	 * <p>
	 * FIXME: this needs a lot of optimization.
	 * </p>
	 * 
	 * @param quickFixTable the table containing the quickfix.
	 * @param quickFixName the name of the quickfix to apply.
	 * @param retries the number of retries, before giving up.
	 * @throws QuickFixNotFoundException if the quickfix could not be found.
	 */
	protected void applyQuickFix(SWTBotTable quickFixTable, String quickFixName, int retries) throws QuickFixNotFoundException {
		if (log.isDebugEnabled())
			log.debug("Trying to activate quickfix shell.");
		int quickFixIndex = quickFixTable.indexOf(quickFixName);
		if (quickFixIndex >= 0) {
			applyQuickFix(quickFixTable, quickFixIndex);
			return;
		}
		throw new QuickFixNotFoundException("Quickfix options not found. Giving up.");
	}

	/**
	 * Gets the quick fix table.
	 * 
	 * @param quickFixShell the shell containing the quickfixes.
	 * @return the table containing the quickfix.
	 * @throws QuickFixNotFoundException if the table could not be found.
	 */
	protected SWTBotTable getQuickFixTable(SWTBotShell quickFixShell) throws QuickFixNotFoundException {
		try {
			return bot.table(quickFixShell);
		} catch (Exception e) {
			throw new QuickFixNotFoundException("Quickfix options not found. Giving up.", e);
		}
	}

	/**
	 * Applies the specified quickfix.
	 * 
	 * @param quickFixTable the table containing the quickfix
	 * @param quickFixIndex the index of the quickfix.
	 */
	protected void applyQuickFix(final SWTBotTable quickFixTable, final int quickFixIndex) {
		asyncExec(new VoidResult() {
			public void run() {
				if (log.isDebugEnabled())
					log.debug("Selecting row [" + quickFixIndex + "] " + getControl().getItem(quickFixIndex).getText() + " in " + widget);
				getControl().setSelection(quickFixIndex);
				Event event = new Event();
				event.type = SWT.Selection;
				event.widget = getControl();
				event.item = getControl().getItem(quickFixIndex);
				getControl().notifyListeners(SWT.Selection, event);
				getControl().notifyListeners(SWT.DefaultSelection, event);
			}

			private Table getControl() {
				return (Table) quickFixTable.widget;
			}
		});
		bot.sleep(1000);
		// quickFixTable.select(quickFixIndex);
	}

	/**
	 * Gets teh active quick fix shell.
	 * 
	 * @return the quickfix shell.
	 * @throws QuickFixNotFoundException if the quickfix shell is not found.
	 */
	protected SWTBotShell activateQuickFixShell() throws QuickFixNotFoundException {
		styledText.notifyKeyboardEvent(SWT.MOD1, '1');
		return activatePopupShell();
	}

	/**
	 * This activates the popup shell.
	 * 
	 * @return The shell.
	 * @throws QuickFixNotFoundException Throw if a quick fix problem occurs.
	 */
	private SWTBotShell activatePopupShell() throws QuickFixNotFoundException {
		try {
			SWTBotShell shell = bot.shell("", 1);
			shell.activate();
			if (log.isDebugEnabled())
				log.debug("Activated quickfix shell.");
			return shell;
		} catch (Exception e) {
			throw new QuickFixNotFoundException("Quickfix popup not found. Giving up.", e);
		}
	}

	/**
	 * Gets the auto completion proposal matching the given text..
	 * 
	 * @param insertText the proposal text to type before auto completing
	 * @return the list of proposals
	 * @throws QuickFixNotFoundException if the autocomplete proposal could not be found.
	 * @throws TimeoutException if the autocomplete shell did not close in time.
	 * @since 1.2
	 */
	public List getAutoCompleteProposals(String insertText) throws QuickFixNotFoundException, TimeoutException {
		styledText.typeText(insertText);
		bot.sleep(1000);
		SWTBotShell autoCompleteShell = activateAutoCompleteShell();
		final SWTBotTable autoCompleteTable = getQuickFixTable(autoCompleteShell);
		List result = syncExec(new ListResult() {
			public List run() {
				TableItem[] items = ((Table) autoCompleteTable.widget).getItems();
				ArrayList result = new ArrayList();
				for (int i = 0; i < items.length; i++) {
					TableItem tableItem = items[i];
					result.add(tableItem.getText());
				}
				return result;
			}
		});
		// makeProposalsDisappear();
		autoCompleteShell.close();
		return result;
	}

	/**
	 * Auto completes the given proposal.
	 * 
	 * @param insertText the text to be inserted before activating the auto-complete.
	 * @param proposalText the auto-completion proposal to select from the list.
	 * @throws QuickFixNotFoundException if the auto-complete proposal could not be found.
	 */
	public void autoCompleteProposal(String insertText, String proposalText) throws QuickFixNotFoundException {
		styledText.typeText(insertText);

		autoComplete(proposalText);
	}

	/**
	 * This performs the auto complete.
	 * 
	 * @param proposalText The text to propose.
	 * @throws QuickFixNotFoundException Thrown if the quick fix error occurs.
	 */
	private void autoComplete(String proposalText) throws QuickFixNotFoundException {
		int retries = 10;
		SWTBotShell autoCompleteShell = activateAutoCompleteShell();
		SWTBotTable autoCompleteTable = getQuickFixTable(autoCompleteShell);
		applyQuickFix(autoCompleteTable, proposalText, retries);
	}

	/**
	 * Gets the active auto complete shell.
	 * 
	 * @return The shell.
	 * @throws QuickFixNotFoundException Thrown if a qaick fix error occurs.
	 */
	private SWTBotShell activateAutoCompleteShell() throws QuickFixNotFoundException {
		styledText.notifyKeyboardEvent(SWT.CTRL, ' ');
		return activatePopupShell();
	}

	/**
	 * Gets the context menu in the editor.
	 * 
	 * @param text the context menu item.
	 * @return the menu
	 * @throws WidgetNotFoundException if the menu with the specified text could not be found.
	 * @see net.sf.swtbot.widgets.AbstractSWTBot#contextMenu(java.lang.String)
	 */
	public SWTBotMenu contextMenu(String text) throws WidgetNotFoundException {
		return styledText.contextMenu(text);
	}

	/**
	 * Gets the current cursor position.
	 * 
	 * @return the position of the cursor.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#cursorPosition()
	 */
	public Position cursorPosition() {
		return styledText.cursorPosition();
	}

	/**
	 * @see net.sf.swtbot.widgets.AbstractSWTBot#isEnabled()
	 */
	public boolean isEnabled() {
		return styledText.isEnabled();
	}

	/**
	 * Gets the current selection.
	 * 
	 * @return The selected string.
	 */
	public String getSelection() {
		return styledText.getSelection();
	}

	/**
	 * Gets the style text.
	 * 
	 * @param line the line number.
	 * @param column the column number.
	 * @return the {@link StyleRange} at the specified location
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#getStyle(int, int)
	 */
	public StyleRange getStyle(int line, int column) {
		return styledText.getStyle(line, column);
	}

	/**
	 * @see net.sf.swtbot.widgets.AbstractSWTBot#getText()
	 */
	public String getText() {
		return styledText.getText();
	}

	/**
	 * @param line the line number.
	 * @param column the column number.
	 * @param text the text to be typed at the specified location
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#typeText(int, int, java.lang.String)
	 * @since 1.0
	 */
	public void typeText(int line, int column, String text) {
		styledText.typeText(line, column, text);
	}

	/**
	 * @param text the text to be typed at the location of the caret. *
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#typeText(java.lang.String)
	 * @since 1.0
	 */
	public void typeText(String text) {
		styledText.typeText(text);
	}

	/**
	 * @param line the line number.
	 * @param column the column number.
	 * @param text the text to be inserted at the specified location
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#insertText(int, int, java.lang.String)
	 */
	public void insertText(int line, int column, String text) {
		styledText.insertText(line, column, text);
	}

	/**
	 * @param text the text to be inserted at the location of the caret.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#insertText(java.lang.String)
	 */
	public void insertText(String text) {
		styledText.insertText(text);
	}

	/**
	 * @param text the text to be typed at the location of the caret.
	 * @param interval the interval between consecutive key strokes.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#typeText(java.lang.String, int)
	 * @since 1.0
	 */
	public void typeText(String text, int interval) {
		styledText.typeText(text, interval);
	}

	/**
	 * @param line the line numnber.
	 * @param column the column number.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#navigateTo(int, int)
	 */
	public void navigateTo(int line, int column) {
		styledText.navigateTo(line, column);
	}

	/**
	 * @param modificationKey the modification key.
	 * @param c the character.
	 * @see Event#character
	 * @see Event#stateMask
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#notifyKeyboardEvent(int, char)
	 */
	public void notifyKeyboardEvent(int modificationKey, char c) {
		styledText.notifyKeyboardEvent(modificationKey, c);
	}

	/**
	 * @param line the line number.
	 * @param column the column number.
	 * @param length the length of the selection.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#selectRange(int, int, int)
	 */
	public void selectRange(int line, int column, int length) {
		styledText.selectRange(line, column, length);
	}

	/**
	 * @param line the line number to select.
	 * @see SWTBotStyledText#selectLine(int)
	 * @since 1.1
	 */
	public void selectLine(int line) {
		styledText.selectLine(line);
	}

	/**
	 * Selects the text on the current line.
	 * 
	 * @see SWTBotStyledText#selectCurrentLine()
	 * @since 1.1
	 */
	public void selectCurrentLine() {
		styledText.selectCurrentLine();
	}

	/**
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#setFocus()
	 */
	public void setFocus() {
		styledText.setFocus();
	}

	/**
	 * @param text the text to set.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#setText(java.lang.String)
	 */
	public void setText(String text) {
		styledText.setText(text);
	}

	/**
	 * Save the editor and close it.
	 */
	public void saveAndClose() {
		save();
		close();
	}

	public void close() {
		syncExec(new VoidResult() {
			public void run() {
				IEditorReference editorReference = (IEditorReference) view;
				editorReference.getPage().closeEditor(editorReference.getEditor(false), false);
			}
		});
	}

	/**
	 * Save the editor.
	 */
	public void save() {
		syncExec(new VoidResult() {
			public void run() {
				((IEditorReference) view).getEditor(false).doSave(null);
			}
		});
	}

	/**
	 * @return the bullet on the current line.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#getBulletOnCurrentLine()
	 */
	public Bullet getBulletOnCurrentLine() {
		return styledText.getBulletOnCurrentLine();
	}

	/**
	 * @param line the line number.
	 * @return the bullet on the given line.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#getBulletOnLine(int)
	 */
	public Bullet getBulletOnLine(int line) {
		return styledText.getBulletOnLine(line);
	}

	/**
	 * @return a copy of the similarWidgets. Changing the returned copy will not affect the copy with this object
	 *         instance.
	 * @see net.sf.swtbot.widgets.AbstractSWTBot#getSimilarWidgets()
	 */
	public List getSimilarWidgets() {
		return styledText.getSimilarWidgets();
	}

	/**
	 * @param line the line number.
	 * @param column the column number.
	 * @param length the length.
	 * @return the styles in the specified range.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#getStyles(int, int, int)
	 */
	public StyleRange[] getStyles(int line, int column, int length) {
		return styledText.getStyles(line, column, length);
	}

	/**
	 * @return the text on the current line, without the line delimiters.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#getTextOnCurrentLine()
	 */
	public String getTextOnCurrentLine() {
		return styledText.getTextOnCurrentLine();
	}

	/**
	 * @param line the line number.
	 * @return the text on the given line number, without the line delimiters.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#getTextOnLine(int)
	 */
	public String getTextOnLine(int line) {
		return styledText.getTextOnLine(line);
	}

	/**
	 * @return <code>true</code> if the styledText has a bullet on the given line, <code>false</code> otherwise.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#hasBulletOnCurrentLine()
	 */
	public boolean hasBulletOnCurrentLine() {
		return styledText.hasBulletOnCurrentLine();
	}

	/**
	 * @param line the line number.
	 * @return <code>true</code> if the styledText has a bullet on the given line, <code>false</code> otherwise.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#hasBulletOnLine(int)
	 */
	public boolean hasBulletOnLine(int line) {
		return styledText.hasBulletOnLine(line);
	}

	/**
	 * @param modificationKey the modification key.
	 * @param c the character.
	 * @param keyCode the keycode.
	 * @see net.sf.swtbot.widgets.SWTBotStyledText#notifyKeyboardEvent(int, char, int)
	 */
	public void notifyKeyboardEvent(int modificationKey, char c, int keyCode) {
		styledText.notifyKeyboardEvent(modificationKey, c, keyCode);
	}

	/**
	 * Shows the editor if it is visible.
	 */
	public void show() {
		syncExec(new VoidResult() {
			public void run() {
				IEditorReference editorReference = (IEditorReference) view;
				IEditorPart editor = editorReference.getEditor(true);
				view.getPage().activate(editor);
			}
		});
	}

}
