/*******************************************************************************
 * Copyright 2007 SWTBot, http://swtbot.org/
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package net.sf.swtbot.eclipse.finder.widgets;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.text.View;

import net.sf.swtbot.SWTBot;
import net.sf.swtbot.eclipse.finder.resolver.EclipseDefaultChildrenResolver;
import net.sf.swtbot.eclipse.finder.resolver.EclipseDefaultParentResolver;
import net.sf.swtbot.finder.ControlFinder;
import net.sf.swtbot.finder.Finder;
import net.sf.swtbot.finder.MenuFinder;
import net.sf.swtbot.finder.UIThreadRunnable;
import net.sf.swtbot.finder.UIThreadRunnable.ListResult;
import net.sf.swtbot.finder.UIThreadRunnable.ObjectResult;
import net.sf.swtbot.finder.UIThreadRunnable.VoidResult;
import net.sf.swtbot.matcher.AllMatcher;
import net.sf.swtbot.matcher.IMatcher;
import net.sf.swtbot.utils.SWTUtils;
import net.sf.swtbot.widgets.AbstractSWTBot;
import net.sf.swtbot.widgets.WidgetNotFoundException;

import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchPartReference;

/**
 * This represents the eclipse {@link View} item.
 * 
 * @author Ketan Padegaonkar &lt;KetanPadegaonkar [at] gmail [dot] com&gt;
 * @version $Id: SWTBotView.java 809 2008-06-29 04:05:10Z kpadegaonkar $
 */
public class SWTBotView extends AbstractSWTBot {

	/** the similar views in eclipse */
	protected List						similarViews;
	/** The view part. */
	protected IWorkbenchPartReference	view;

	/**
	 * Constructs a new instance of this with the given finder and text to search for.
	 * 
	 * @param finder the finder used to find controls.
	 * @param text the text on the control.
	 * @throws WidgetNotFoundException if the widget is not found.
	 */
	public SWTBotView(Finder finder, String text) throws WidgetNotFoundException {
		super(finder, text);
	}

	/**
	 * This constructor calls {@link #findControls(IMatcher)} to find all controls matching {@link #matcher()}, in case
	 * a widget is not found, the constructor retries until a timeout occurs. The first such control is set into
	 * {@link #widget} the other controls, if any that match {@link #matcher()} are held in the {@link #similarWidgets}
	 * list.
	 * 
	 * @see SWTBot#waitUntil(net.sf.swtbot.wait.ICondition)
	 * @param finder the finder used to find controls.
	 * @param text the text on the control.
	 * @param index the index of the control in case multiple controls are found.
	 * @throws WidgetNotFoundException if the widget is not found.
	 * @since 1.0
	 */
	public SWTBotView(Finder finder, String text, int index) throws WidgetNotFoundException {
		super(finder, text, index);
	}

	/**
	 * Constructs an instance of the view with the given reference.
	 * 
	 * @param reference the view reference.
	 * @throws WidgetNotFoundException if the widget is <code>null</code> or widget has been disposed.
	 */
	public SWTBotView(IWorkbenchPartReference reference) throws WidgetNotFoundException {
		super(findWidget(reference, new Finder(new ControlFinder(new EclipseDefaultChildrenResolver(), new EclipseDefaultParentResolver()),
				new MenuFinder())));
		view = reference;
	}

	/**
	 * @see net.sf.swtbot.widgets.AbstractSWTBot#getMatcher()
	 */
	protected IMatcher getMatcher() {
		return net.sf.swtbot.matcher.WidgetMatcherFactory.cTabItemMatcher(text);
	}

	/**
	 * Finds the widget at the given index.
	 * 
	 * @since 1.0
	 */
	protected Widget findWidget(int index) throws WidgetNotFoundException {
		if (view == null)
			view = (IWorkbenchPartReference) similarViews.get(index);
		return findWidget(view, finder);
	}

	/**
	 * Finds the widget using the given view and finder.
	 * 
	 * @param view the part reference.
	 * @param finder the finder to be used to find controls in the view.
	 * @return the widget in the part reference.
	 * @throws WidgetNotFoundException if the widget is not found.
	 */
	protected static Widget findWidget(IWorkbenchPartReference view, Finder finder) throws WidgetNotFoundException {
		if (view instanceof WorkbenchPartReference) {
			Control control = ((WorkbenchPartReference) view).getPane().getControl();
			boolean shouldFindInvisibleControls = finder.shouldFindInvisibleControls();
			finder.setShouldFindInvisibleControls(true);
			try {
				return (Widget) finder.findControls(control, new AllMatcher(), true).get(1);
			} catch (Exception e) {
				throw new WidgetNotFoundException("Could not find any control inside the view " + view.getPartName(), e);
			} finally {
				finder.setShouldFindInvisibleControls(shouldFindInvisibleControls);
			}
		}
		throw new WidgetNotFoundException("Could not find any control inside the view " + view.getPartName());
	}

	/**
	 * @see net.sf.swtbot.widgets.AbstractSWTBot#findControls(net.sf.swtbot.matcher.IMatcher)
	 */
	protected List findControls(IMatcher matcher) {
		similarViews = filterResults(allViews());
		return super.findControls(matcher);
	}

	/**
	 * Gets all the view available.
	 * 
	 * @return all the views in the workbench
	 */
	public static List allViews() {
		ArrayList result = new ArrayList();
		result.addAll(findViews());
		result.addAll(findEditors());
		return result;
	}

	/**
	 * Gets a list of items using the filters.
	 * 
	 * @param toFilter the list of workbench part references to be filtered.
	 * @return the parts that have the name as that in {@link #text}
	 */
	protected ArrayList filterResults(List toFilter) {
		ArrayList matches = new ArrayList();

		for (Iterator iterator = toFilter.iterator(); iterator.hasNext();) {
			IWorkbenchPartReference workbenchPart = (IWorkbenchPartReference) iterator.next();
			if (workbenchPart.getPartName().equals(text))
				matches.add(workbenchPart);
		}
		return matches;
	}

	/**
	 * Finds the editors.
	 * 
	 * @return all the editors in the workbench.
	 */
	public static List findEditors() {
		return UIThreadRunnable.syncExec(SWTUtils.display(), new ListResult() {
			public List run() {
				ArrayList result = new ArrayList();
				IWorkbenchPage[] pages = getWorkbenchPages();
				for (int i = 0; i < pages.length; i++) {
					IWorkbenchPage page = pages[i];
					IEditorReference[] editorReferences = page.getEditorReferences();
					for (int j = 0; j < editorReferences.length; j++) {
						IEditorReference editorReference = editorReferences[j];
						result.add(editorReference);
					}
				}
				return result;
			}
		});
	}

	/**
	 * gets the workbench pages.
	 * 
	 * @return The {@link IWorkbenchPage} array.
	 */
	private static IWorkbenchPage[] getWorkbenchPages() {
		IWorkbench workbench = PlatformUI.getWorkbench();
		IWorkbenchWindow activeWorkbenchWindow = workbench.getActiveWorkbenchWindow();
		IWorkbenchPage[] pages = activeWorkbenchWindow.getPages();
		return pages;
	}

	/**
	 * Finds all the views.
	 * 
	 * @return all the views in the workbench.
	 */
	public static List findViews() {
		return UIThreadRunnable.syncExec(SWTUtils.display(), new ListResult() {
			public List run() {
				ArrayList result = new ArrayList();
				IWorkbenchPage[] pages = getWorkbenchPages();
				for (int i = 0; i < pages.length; i++) {
					IWorkbenchPage page = pages[i];
					IViewReference[] viewReferences = page.getViewReferences();
					for (int j = 0; j < viewReferences.length; j++) {
						IViewReference viewReference = viewReferences[j];
						result.add(viewReference);
					}
				}
				return result;
			}
		});
	}

	/**
	 * Gets the active editor.
	 * 
	 * @return the active editor, if any
	 * @throws WidgetNotFoundException if there is no active editor.
	 * @since 1.1
	 */
	public static SWTBotEclipseEditor activeEditor() throws WidgetNotFoundException {
		Object editor = UIThreadRunnable.syncExec(new ObjectResult() {
			public Object run() {
				try {
					IEditorPart activeEditor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
					return PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getReference(activeEditor);
				} catch (RuntimeException e) {
					return null;
				}
			}
		});
		if (editor == null)
			throw new WidgetNotFoundException("There is no active editor");
		return new SWTBotEclipseEditor((IEditorReference) editor);
	}

	/**
	 * Close the view.
	 */
	public void close() {
		syncExec(new VoidResult() {
			public void run() {
				IViewReference viewReference = (IViewReference) view;
				viewReference.getPage().hideView(viewReference);
			}
		});
	}

	/**
	 * Shows the part if it is visible.
	 * 
	 * @since 1.0
	 */
	public void show() {
		syncExec(new VoidResult() {
			public void run() {
				try {
					view.getPage().activate(view.getPart(true));
					view.getPage().showView(view.getId());
				} catch (PartInitException e) {
					throw new RuntimeException("Could not show view - " + view.getPartName(), e);
				}
			}
		});
	}

	/**
	 * Gets the title of the view.
	 * 
	 * @return the title of the part as visible in the tab
	 * @since 1.2
	 */
	public String getTitle() {
		return view.getPartName();
	}
}
